import fs from "fs";
import fetch from "node-fetch";
import sharp from "sharp";
import { dirname } from "path";
import { fileURLToPath } from "url";

const __dirname = dirname(fileURLToPath(import.meta.url));

const bakeImage = async (url, size, saveStr, tagStr = "") => {
  const width = size;
  const height = size;
  const margin = width * 0.03;

  const saveBoxSize = size * 0.21;
  const saveBoxPadding = saveBoxSize * 0.1;
  const saveFontSize = size * 0.055;
  const saveFontSize2 = size * (3.5 / saveStr.length) * 0.055;
  const saveBoxX = width - saveBoxSize - margin;
  const saveRadius = 20;

  const tagFontSize = size * 0.04;
  const tagBoxWidth = tagFontSize * tagStr.length * 0.9;
  const tagBoxHeight = tagFontSize * 1.5;
  const tagBoxMargin = margin;
  const tagBoxX = width - tagBoxWidth - tagBoxMargin;
  const tagBoxY = height - tagBoxHeight - tagBoxMargin;

  const font64 = fs
    .readFileSync(__dirname + "/Montserrat-Bold.ttf")
    .toString("base64");

  const fontStyle = `
    <style>
    @font-face {
        font-family: "Montserrat";
        src: url("data:application/font-woff;charset=utf-8;base64,${font64}");
    }
    </style>
  `;

  const saveFragment = `
  <rect
    x="${saveBoxX}"
    y="${-saveRadius}"
    height="${saveBoxSize + saveRadius}"
    width="${saveBoxSize}"
    fill="#E50E62"
    rx="${saveRadius}"
  />

  <text text-anchor="middle" x="${saveBoxX + saveBoxSize / 2}" y="${
    saveFontSize + saveBoxPadding
  }px" font-weight="bold" font-family="Montserrat" font-size="${saveFontSize}" fill="white">SAVE</text>

  <text text-anchor="middle" x="${saveBoxX + saveBoxSize / 2}" y="${
    saveFontSize * 2.5 + saveBoxPadding
  }px" font-weight="bold" font-family="Montserrat" font-size="${saveFontSize2}" fill="white">${saveStr}</text>
  `;

  let tagFragment = "";

  if (tagStr) {
    tagFragment = `
    <rect
      x="${tagBoxX}"
      y="${tagBoxY}"
      height="${tagBoxHeight}"
      width="${tagBoxWidth}"
      fill="white"
      rx="30"
      filter="url(#shadow-filter)"
    />
    
    <text x="${tagBoxX + tagBoxWidth / 2}" y="${
      tagBoxY + tagBoxHeight / 2
    }" fill="#757F8C" text-anchor="middle" dy=".35em" font-weight="bold" font-family="Montserrat" font-size="${tagFontSize}" letter-spacing="1.1">
      ${tagStr}
    </text>
    `;
  }

  const svg = `
  <svg
    viewBox="0 0 ${width} ${height}"
    version="1.1"
    baseProfile="full"
    xmlns="http://www.w3.org/2000/svg"
  >
    <defs>
      <filter x="-50%" y="-50%" width="200%" height="200%" filterUnits="objectBoundingBox" id="shadow-filter">
        <feOffset dx="0" dy="4" in="SourceAlpha" result="shadowOffsetOuter1"/>
        <feGaussianBlur stdDeviation="10" in="shadowOffsetOuter1" result="shadowBlurOuter1"/>
        <feColorMatrix values="0 0 0 0 0   0 0 0 0 0   0 0 0 0 0  0 0 0 0.2 0" in="shadowBlurOuter1" type="matrix" result="shadowMatrixOuter1"/>
        <feMerge>
          <feMergeNode in="shadowMatrixOuter1"/>
          <feMergeNode in="SourceGraphic"/>
        </feMerge>
      </filter>
    </defs>
    ${fontStyle}
    <rect
      x="0"
      y="0"
      height="${height}"
      width="${width}"
      fill="transparent"
    />
  
    ${saveFragment}
    ${tagFragment}
  </svg>
  `;

  const response = await fetch(url);
  const data = await response.buffer();

  return sharp(data)
    .resize(width, height)
    .composite([
      {
        input: Buffer.from(svg),
        blend: "over",
      },
    ])
    .jpeg()
    .toBuffer();
};

export default bakeImage;
