import { ImageUrls } from '../image/imageUrls';
import { Round10, Floor10, Ceiling10 } from '../image/mathUtility';

export const Content = {

  unpack(message) {
    return this._translate(JSON.parse(message));
  },

  _translate(jsonMessage) {

    let addContentMessage = {};

    addContentMessage.id = jsonMessage.url;

    addContentMessage.key = "url";

    addContentMessage.keys = {};
    addContentMessage.keys.sku = jsonMessage.vars.product_id;

    addContentMessage.title = jsonMessage.title;

    if(jsonMessage.description.description) {
      addContentMessage.description = jsonMessage.description.description;
    } else if (jsonMessage.description.short_description) {
      addContentMessage.description = jsonMessage.description.short_description;
    } else {
      addContentMessage.description = jsonMessage.description.more_detail;
    }
     
    /* on magento side structure:
    "description": {
      "description": "",
      "short_description": "",
      "more_detail": //holds most of description if not all
    }
    */

    addContentMessage.price = jsonMessage.vars.price;
    addContentMessage.inventory = jsonMessage.inventory;
    addContentMessage.images = this._createAdditionalUrls(jsonMessage.images, 
      jsonMessage.vars.display_savings_in_rands, jsonMessage.vars.price, jsonMessage.vars.retail);
    addContentMessage.tags = "";

    addContentMessage.date = jsonMessage.vars.special_from_date; 
    addContentMessage.expire_date = jsonMessage.expire_date; 
    // not vars.special_to_date which is the same often as special_from_date

    addContentMessage.vars = jsonMessage.vars;
    
    addContentMessage.vars.description = jsonMessage.description.description;
    addContentMessage.vars.short_description = jsonMessage.description.short_description;
    addContentMessage.vars.more_detail_description = jsonMessage.description.more_detail;

    addContentMessage.vars.display_message_from_tags = "";

    if (typeof jsonMessage.vars.blue_tag != "undefined" && 
      jsonMessage.vars.blue_tag != null && jsonMessage.vars.blue_tag.length > 0) {
        addContentMessage.vars.display_message_from_tags = jsonMessage.vars.blue_tag;
    } else if(typeof jsonMessage.vars.yellow_tag != "undefined" && 
      jsonMessage.vars.yellow_tag != null && jsonMessage.vars.yellow_tag.length > 0) {
        addContentMessage.vars.display_message_from_tags = jsonMessage.vars.yellow_tag;
    }

    addContentMessage.override_exclude = 1;

    return addContentMessage;
  },

  _createAdditionalUrls(images, isSavingInRands, price, retail) {

  const intPrice = parseInt(price);
  const intRetail = parseInt(retail);

  let savings = "";
  if (isSavingInRands == "y") {
    let preRoundedSavings = ((intRetail-intPrice)*0.01).toFixed();
    savings = `R${Floor10(preRoundedSavings, 1)}`;
  } else {
    savings = `${((1 - intPrice/intRetail)*100).toFixed()}%`;
  }

  images['thumb_120x120'] = {
    "url" : ImageUrls.addScaleOnly(images.thumb.url, 120, 120)
  };

  images['email_thumb_120x120'] = {
    "url" : ImageUrls.addScaleOnly(images.email_thumb.url, 120, 120)
  };

  images['thumb_240x240'] = {
    "url" : ImageUrls.addScaleOnly(images.thumb.url, 240, 240)
  };

  images['email_thumb_240x240'] = {
    "url" : ImageUrls.addScaleOnly(images.email_thumb.url, 240, 240)
  };

  images['email_240x240_savings'] = {
    "url" : ImageUrls.addSavingsBlockAndScale(images.email.url, 240, 240, savings)
  };

  images['email_365x365_savings'] = {
    "url" : ImageUrls.addSavingsBlockAndScale(images.email.url, 365, 365, savings)
  };

  images['email_480x480_savings'] = {
    "url" : ImageUrls.addSavingsBlockAndScale(images.email.url, 480, 480, savings)
  };

  images['email_750x300_savings'] = {
    "url" : ImageUrls.addSavingsBlockAndScale(images.email.url, 750, 'auto', savings)
  };

  return images;

  }

}